



#include "RHB_Custom.h"




ClassImp (RHB_Custom)



void  RHB_Custom::InitParams (void) {

  FEventProcessorThread::InitParams ();       // call official function
                                              // and add user specific part :

  this->signal    = GetParam ("CH1_SAMPLES"); // get a param which already exists (basic SAMPLER data)

  this->qdc_tot   = new FParamBrutSimple ();  // create six new params available in the facqonf file
  this->qdc_fast  = new FParamBrutSimple ();
  this->qdc_ratio = new FParamBrutSimple ();
  this->unknowns  = new FParamBrutSimple ();
  this->neutrons  = new FParamBrutSimple ();
  this->gammas    = new FParamBrutSimple ();

  this->qdc_tot->SetName   ("Qtot");          // name of the new parameters in SAMPLER.facqconf
  this->qdc_fast->SetName  ("Qfast");
  this->qdc_ratio->SetName ("Qratio");
  this->unknowns->SetName  ("Unknowns");
  this->neutrons->SetName  ("Neutrons");
  this->gammas->SetName    ("Gammas");

  this->AddRawParam        (this->qdc_tot);   // add those six params
  this->AddRawParam        (this->qdc_fast);
  this->AddRawParam        (this->qdc_ratio);
  this->AddRawParam        (this->unknowns);
  this->AddRawParam        (this->neutrons);
  this->AddRawParam        (this->gammas);

  this->nu = 0;                               // internal variables initialisation
  this->nn = 0;
  this->ng = 0;
  this->nb_fast      = 30;
  this->charge_thres = 15000.0;
  this->ratio_thres  = 0.85;

}





void RHB_Custom::SetParamValues (void) {

  //  local variables
  Double_t qtot  =  0.0;
  Double_t qfast =  0.0;
  Double_t ratio = -1.0;
  Double_t val   =  0.0;
  Int_t    npts  =  0;

  FEventProcessorThread::SetParamValues ();                   //  call official function

                                                              //  add user specific part :

  if (this->signal->IsOn ()) {                                //  when data is ch1 sampler
    npts = this->signal->GetNbValues ();                      //    - get number of samples
    for (int i=0; i<npts; i++) {                              //    - loop on each sample of the signal
      val   = this->signal->GetValue (i);                     //       - get the value of the sample
      qtot += val;                                            //       - calculate total charge (on all values)
      if (i < this->nb_fast) qfast += val;                    //       - calculate fast charge  (on the first nb_fast values)
    }                                                         //
    if (qtot != 0.0) ratio = qfast/qtot;                      //    - calculate charge ratio

                                                              //    - update custom rhb parameters
    this->qdc_tot->SetValue  (qtot);                          //       - Qtot
    this->qdc_fast->SetValue (qfast);                         //       - Qfast
    this->qdc_ratio->SetValue (ratio);                        //       - Qratio
    if (   (qtot < this->charge_thres)
        || (ratio > 1.0)
        || (ratio < 0.0)) {
      this->nu += 1;                                          //       - Unknowns
      this->unknowns->SetValue (this->nu);                    //
    } else if (ratio < this->ratio_thres) {
      this->nn += 1;                                          //       - Neutrons
      this->neutrons->SetValue (this->nn);                    //
    } else {
      this->ng += 1;                                          //       - Gammas
      this->gammas->SetValue (this->ng);                      //
    }
  }

}



