



#include "RHB_Custom.h"




ClassImp (RHB_Custom)



void  RHB_Custom::InitParams (void) {

  FEventProcessorThread::InitParams ();       // call official function
                                              // and add user specific part :

  this->signal    = GetParam ("CH1_SAMPLES"); // get a param already declared (in .pid)

  this->nb_pts    = new FParamBrutSimple ();  // create a new custom param
  this->base_line = new FParamBrutSimple ();  // create a new custom param
  this->charge    = new FParamBrutSimple ();  // ...
  this->max       = new FParamBrutSimple ();
  this->min       = new FParamBrutSimple ();
  this->fwhm      = new FParamBrutSimple ();
  this->risetime  = new FParamBrutSimple ();
  this->nb_pts->SetName    ("NbPts");         // name of that param in .facqConf
  this->base_line->SetName ("BaseLine");      // ...
  this->charge->SetName    ("Charge");
  this->max->SetName       ("Max");
  this->min->SetName       ("Min");
  this->fwhm->SetName      ("Fwhm");
  this->risetime->SetName  ("RiseTime");      // add the param
  this->AddRawParam (this->nb_pts);           // ...
  this->AddRawParam (this->base_line);
  this->AddRawParam (this->charge);
  this->AddRawParam (this->max);
  this->AddRawParam (this->min);
  this->AddRawParam (this->fwhm);
  this->AddRawParam (this->risetime);

}





void RHB_Custom::SetParamValues (void) {

  //  local variables
  Double_t val   =  0.0;
  Double_t max   = -10000.0;
  Double_t min   =  10000.0;
  Double_t q     =  0.0;
  Double_t bl    =  0.0;
  Int_t    nbl   =  10;
  Int_t    n_max =  0;
  Int_t    n_hm1 =  0;
  Int_t    n_hm2 =  0;
  Int_t    n_rt1 =  0;
  Int_t    n_rt2 =  0;
  Int_t    npts  =  0;

  FEventProcessorThread::SetParamValues ();                //  call official function
                                                           //  and add user specific part :
                                                           //
  if (this->signal->IsOn ()) {                             //  when data is ch1 sampler data
    npts = this->signal->GetNbValues ();                   //    - get number of samples
    for (int i=0; i<npts; i++) {                           //    - iterate on the samples of the signal
      val = this->signal->GetValue (i);                    //         - get the value of the sample
      q += val;                                            //         - calculate the charge q (sum of all the samples)
      if (i   < nbl) bl += val;                            //         - calculate the base line (sum of the first samples)
      if (val < min) min = val;                            //         - search for the value minimum
      if (val > max) {                                     //         - search for the max value and its position within the signal
        max   = val;
        n_max = i;
      }
    }
    bl    = bl / nbl;                                      //    - average the base line
    max   = max - bl;                                      //    - adjust min, max and q relative to base line
    min   = min - bl;
    q     = q   - bl * npts;
    n_hm1 = 0;
    n_hm2 = npts - 1;
    n_rt1 = 0;
    n_rt2 = npts - 1;
    for (int i=n_max; i<npts; i++) {                       //    - look for the fwhm index
      val = this->signal->GetValue (i) - bl;               //      ...
      if (val <= max / 2) {
        n_hm2 = i;
        break;
      }
    }
    for (int i=n_max; i>=0; i--) {                         //      ...
      val = this->signal->GetValue (i) - bl;               //
      if (val <= max / 2) {
        n_hm1 = i;
        break;
      }
    }
    for (int i=n_max; i>=0; i--) {                         //    - look for the rise time index
      val = this->signal->GetValue (i) - bl;               //
      if (val <= max * 0.9) {
        n_rt2 = i;
        break;
      }
    }
    for (int i=n_rt2; i>=0; i--) {                         //      ...
      val = this->signal->GetValue (i) - bl;               //
      if (val <= max * 0.1) {
        n_rt1 = i;
        break;
      }
    }
    this->nb_pts->SetValue    (npts);                      //    - update custom parameters
    this->base_line->SetValue (bl);                        //    - ...
    this->charge->SetValue    (q);                         //    - ...
    this->max->SetValue       (max);                       //    - ...
    this->min->SetValue       (min);                       //    - ...
    this->fwhm->SetValue      (n_hm2 - n_hm1);             //    - fwhm (nb of pts) and update coresponding custom param
    this->risetime->SetValue  (n_rt2 - n_rt1);             //    - rise time (nb of pts) and update coresponding custom param
  }

}



